        /*

        Nanode + Pachube + DS18*20 temperature sensor + relay = internet controlled thermostat
        
        Code hacked together by krekr (gregor@krekr.nl)
        
        Code relies heavily on the following arduino sketches:
        
        EtherShield ENC ReadPachube example
        PacubeV3_WithoutNode by WickedDevice (http://blog.wickeddevice.com/?p=260)
        The Dallas Temperature library and its examples (http://milesburton.com/Dallas_Temperature_Control_Library)
        
        
        Features:
        DHCP
        DNS
        Onewire temp sensing
                
        
        
        Todo:
        PID control
        Local Override
        Failsafe (no connection, weird data, ...)
        Neater temperature getting from pachube ("Z,")
        Pachube normal temperatures (no *100; floats instead)
        Timing (programming) (better yet: self programming)
        
        Scheme:
        
        Nanode or Arduino with ENC ethernet shield
        Relay on pin 5
        OneWire temp sensor (DS18B20) on pin 4
       
        

         */
        
        #include "EtherShield.h"  
        #include <OneWire.h>
      #include <DallasTemperature.h>
      
      
        
        /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
         * The following #defines govern the behavior of the sketch. You can console outputs using the Serial Monitor
         * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
        
        #define MY_MAC_ADDRESS {0x54,0x55,0x58,0x10,0x00,0x25}               // must be uniquely defined for all Nanodes, e.g. just change the last number
        #define USE_DHCP                                                     // comment out this line to use static network parameters
        #define PACHUBE_API_KEY "*********************************" // change this to your API key
        #define HTTPFEEDPATH "/v2/feeds/39664"                               // change this to th relative URL of your feed

        #define DELAY_BETWEEN_PACHUBE_POSTS_MS 30000L       // up from 15 secs
        #define SERIAL_BAUD_RATE 9600
        
        #ifndef USE_DHCP // then you need to supply static network parameters, only if you are not using DHCP
          #define MY_IP_ADDRESS {192,168,  0, 25}
          #define MY_NET_MASK   {255,255,255,  0}
          #define MY_GATEWAY    {192,168,  0,  1}
          #define MY_DNS_SERVER {192,168,  0,  1}
        #endif
        
        // change the template to be consistent with your datastreams: see http://api.pachube.com/v2/
        #define FEED_POST_MAX_LENGTH 256
        static char feedTemplate[] = "{\"version\":\"1.0.0\",\"datastreams\":[{\"id\":\"ActualTemp\", \"current_value\":\"%d\"}]}";
        static char feedPost[FEED_POST_MAX_LENGTH] = {0}; // this will hold your filled out template
        uint8_t fillOutTemplateWithSensorValues(uint16_t node_id, uint16_t sensorValue1, uint16_t sensorValue2, uint16_t sensorValue3, uint16_t sensorValue4){
          // change this function to be consistent with your feed template, it will be passed the node id and four sensor values by the sketch
          // if you return (1) this the sketch will post the contents of feedPost to Pachube, if you return (0) it will not post to Pachube
          // you may use as much of the passed information as you need to fill out the template
          
          snprintf(feedPost, FEED_POST_MAX_LENGTH, feedTemplate, sensorValue1, sensorValue2); // this simply populates the current_value filed with sensorValue1
          return (1);
        }
        
        /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
         * You shouldn't need to make changes below here for configuring the sketch
         * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
        
        // mac and ip (if not using DHCP) have to be unique
        // in your local area network. You can not have the same numbers in
        // two devices:
        static uint8_t mymac[6] = MY_MAC_ADDRESS;
        
        // IP address of the host being queried to contact (IP of the first portion of the URL):
        static uint8_t websrvip[4] = { 0, 0, 0, 0 }; // resolved through DNS
        
        #ifndef USE_DHCP
        // use the provided static parameters
        static uint8_t myip[4]      = MY_IP_ADDRESS;
        static uint8_t mynetmask[4] = MY_NET_MASK;
        static uint8_t gwip[4]      = MY_GATEWAY;
        static uint8_t dnsip[4]     = MY_DNS_SERVER;
        #else
        // these will all be resolved through DHCP
        static uint8_t dhcpsvrip[4] = { 0,0,0,0 };    
        static uint8_t myip[4]      = { 0,0,0,0 };
        static uint8_t mynetmask[4] = { 0,0,0,0 };
        static uint8_t gwip[4]      = { 0,0,0,0 };
        static uint8_t dnsip[4]     = { 0,0,0,0 };
        #endif
        
        long lastPostTimestamp;
        
 
        boolean firstTimeFlag = true;
        
        boolean getDone = false; //are we done reading from pachube?
        
        
        //for posting to pachube
        #define FEEDHOSTNAME "api.pachube.com\r\nX-PachubeApiKey: " PACHUBE_API_KEY
        #define FEEDWEBSERVER_VHOST "api.pachube.com"
        
        static char hoststr[150] = FEEDWEBSERVER_VHOST;
        
        #define BUFFER_SIZE 550
        static uint8_t buf[BUFFER_SIZE+1];

        
        EtherShield es=EtherShield();
     
        
        // for getting from pachube
        #define PORT 80                   // HTTP        
        #define HOSTNAME "api.pachube.com\r\nX-PachubeApiKey: **************************"      // API key
      //  static uint8_t websrvip[4] = { 0,0,0,0 };	// Get pachube ip by DNS call
        #define WEBSERVER_VHOST "api.pachube.com"
        #define HTTPPATH "/v2/feeds/39664.csv?datastreams=RequestTemp"      // Set your own feed ID here
      
      
      
      
      // Data wire is plugged into port 4 on the Arduino
      #define ONE_WIRE_BUS 4
      // Setup a oneWire instance to communicate with any OneWire devices (not just Maxim/Dallas temperature ICs)
      OneWire oneWire(ONE_WIRE_BUS);
      // Pass our oneWire reference to Dallas Temperature. 
      DallasTemperature sensors(&oneWire);
      // arrays to hold device address
      DeviceAddress insideThermometer;
        
      
        float actualTemp; //the temperature we measure
        
        int requestTemp = 0; // the user requested temperature (setpoint)
        
        int relayPin = 5; 
        
        long lastSwitchTimestamp = 0; //when have we last turned heating on or off - to prevent hysteresis
        
        long minSwitchTime = 5 * 60 * 1000; //1 minute min switch time
        
        
        
        
        
  void setup(){
          Serial.begin(SERIAL_BAUD_RATE);
          Serial.println("Nanode + Pachube + DS18*20 temperature sensor + relay = internet controlled thermostat");
        
        
          sensors.begin();
          if (!sensors.getAddress(insideThermometer, 0)) Serial.println("Unable to find address for Device 0"); 
      
        // set the resolution to 12 bit: better resolution 
        sensors.setResolution(insideThermometer, 12);
      
        
        
          // Initialise SPI interface
          es.ES_enc28j60SpiInit();
        
          // initialize ENC28J60
          es.ES_enc28j60Init(mymac, 8);
        
        #ifdef USE_DHCP
          acquireIPAddress();
        #endif
        
          printNetworkParameters();
        
          //init the ethernet/ip layer:
          es.ES_init_ip_arp_udp_tcp(mymac,myip, 80);
        
          // init the web client:
          es.ES_client_set_gwip(gwip);  // e.g internal IP of dsl router
          es.ES_dnslkup_set_dnsip(dnsip); // generally same IP as router
          
          Serial.println("Awaiting Client Gateway");
          while(es.ES_client_waiting_gw()){
            int plen = es.ES_enc28j60PacketReceive(BUFFER_SIZE, buf);
            es.ES_packetloop_icmp_tcp(buf,plen);    
          }
          Serial.println("Client Gateway Complete, Resolving Host");
        
          resolveHost(hoststr, websrvip);
          Serial.print("Resolved host: ");
          Serial.print(hoststr);
          Serial.print(" to IP: ");
          printIP(websrvip);
          Serial.println();
          
          es.ES_client_set_wwwip(websrvip);
          
          lastPostTimestamp = millis();
          
          //Relay is an output; default off
          pinMode(relayPin, OUTPUT);
          digitalWrite(relayPin, LOW);
          
        }//setup
        
        
        
      
      
      
        
      
        
  void loop(){
      
         
      
          long currentTime = millis(); //timer which is used to trigger actions
          
          //Start receiving packets and put them in buffer
          int plen = es.ES_enc28j60PacketReceive(BUFFER_SIZE, buf);
          es.ES_packetloop_icmp_tcp(buf,plen);
          
          
          //posting to pachube
          if(currentTime - lastPostTimestamp > DELAY_BETWEEN_PACHUBE_POSTS_MS || firstTimeFlag){   
            firstTimeFlag = false;
            uint16_t sensorValue1 = 0; //these are obsolete
            uint16_t sensorValue2 = 0;
            uint16_t sensorValue3 = 0;
            uint16_t sensorValue4 = 0;  
         
         
            sensors.requestTemperatures(); // Send the command to get temperatures
        
      
            actualTemp = sensors.getTempC(insideThermometer); //get temperature from DS18
              
              uint16_t actualTemp100 = actualTemp * 100; //Pachube doesn't understand floats from my nanode, so make it centi-degrees-celcius (20C = 2000cC)
      
            
            if(fillOutTemplateWithSensorValues(0, actualTemp100, sensorValue2, sensorValue3, sensorValue4)){ //if filling out the template is succesful, post it.
              Serial.print("Posting sensor values to Pachube: ");
              Serial.print(actualTemp, DEC);
              Serial.println("");

              
              es.ES_client_http_post(PSTR(HTTPFEEDPATH),PSTR(FEEDWEBSERVER_VHOST),PSTR(FEEDHOSTNAME), PSTR("PUT "), feedPost, &sensor_feed_post_callback); //sensor_feed_post_callback   
              
            
            }
            lastPostTimestamp = currentTime;
            
            getDone = false; //toggle the "getting" flag back
          }
          
          
          //getting from pachube
              if(currentTime - lastPostTimestamp > 5000 && firstTimeFlag == false && getDone == false){   // 5 secs after posting to pachube

                  es.ES_client_browse_url(PSTR(HTTPPATH), NULL, PSTR(HOSTNAME), &browserresult_callback); //, get requesttemp and call browserresult_callback
                  
                  getDone = true; //we're done
                  
                }
          
          
          
          //doing actual stuff:
          
         if ( requestTemp > actualTemp && currentTime - lastSwitchTimestamp >= minSwitchTime) //then it's too cold! 
         
         {
           
           digitalWrite(relayPin, HIGH);
           
           lastSwitchTimestamp = currentTime;
           
         }

         if ( requestTemp < actualTemp && currentTime - lastSwitchTimestamp >= minSwitchTime) //then it's too hot!
         
         {
           
           digitalWrite(relayPin, LOW);
          
           lastSwitchTimestamp = currentTime;
           
         }          
          
          
          
        } //loop
        
        
        
        
        
        
              int Str2int (String Str_value) //converts a string (up to 3 characters) to an int. used to interpret requesttemp from pachube
                  {
                    char buffer[3]; //max length is three units
                    Str_value.toCharArray(buffer, 3);
                    int int_value = atoi(buffer);
                    return int_value;
                  }
                  
                  
                  
      void browserresult_callback(uint8_t statuscode, uint16_t datapos, uint16_t jemoeder){
          if (datapos != 0)
          {
            // now search for the csv data - it follows the first blank line
            // I'm sure that there is an easier way to search for a blank line - but I threw this together quickly
            // and it works for me.
            uint16_t pos = datapos;
            while (buf[pos])    // loop until end of buffer (or we break out having found what we wanted)
            {
              while (buf[pos]) if (buf[pos++] == '\n') break;   // find the first line feed
              if (buf[pos] == 0) break; // run out of buffer
              if (buf[pos++] == '\r') break; // if it is followed by a carriage return then it is a blank line (\r\n\r\n)
            }
            if (buf[pos])  // we didn't run out of buffer
            {
              pos++;  //skip over the '\n' remaining
              
 
              String result = String( ( char* ) & buf[pos]);   //get everything in buffer and make it a string
              String query = String("Z,");                     //search for "Z,", after which the actual value we want starts. Might stop working!
              String requestTempStr = result.substring(result.lastIndexOf(query) + 2); //remove everything except the value
                  
              requestTemp = Str2int (requestTempStr);         //convert to int
              
              Serial.print("Temperature Requested: ");
              Serial.println( requestTemp );
              
            }
          }
      
      }                  
